unit MainWindU;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ControllerU, TimerU, ExtCtrls;

type
  TMainWindow = class(TForm)
    GoTimer: TTimer;
    PB: TPaintBox;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure GoTimerTimer(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure PBPaint(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
  private
    { Private declarations }
    FController: TController;
    FBackBuffer: TBitmap;
    FTimer: TMSTimer;
    FDone: Boolean;
    procedure MainLoop;
  public
    { Public declarations }
  end;

var
  MainWindow: TMainWindow;

implementation

uses ParamsU;

{$R *.dfm}

procedure TMainWindow.FormCreate(Sender: TObject);
begin
   Self.ControlStyle := Self.ControlStyle + [csOpaque];

   Width := Params.WindowWidth;
   Height := Params.WindowHeight;
   Self.Caption := Application.Title;
   FDone := False;

   FTimer := TMSTimer.Create(Params.FramesPerSecond);
   FTimer.Start;

   FBackBuffer := TBitmap.Create;
   FBackBuffer.Width := Self.ClientWidth;
   FBackBuffer.Height := Self.ClientHeight;
   FController := TController.Create;
end;

procedure TMainWindow.FormDestroy(Sender: TObject);
begin
   FTimer.Free;
   FController.Free;
   FBackBuffer.Free;
end;

procedure TMainWindow.GoTimerTimer(Sender: TObject);
begin
   GoTimer.Enabled := False;
   MainLoop;
end;

procedure TMainWindow.MainLoop;
begin
   While not FDone do
   begin
      if FTimer.ReadyForNextFrame or FController.FastRender then
      begin
         FController.Update;

         InvalidateRect(Self.Handle, nil, false);
         UpdateWindow(Self.Handle);
      end;
      Application.ProcessMessages;
   end;
end;

procedure TMainWindow.FormClose(Sender: TObject; var Action: TCloseAction);
begin
   FDone := True;
end;

procedure TMainWindow.PBPaint(Sender: TObject);
var
   r: TRect;
begin
   r := PB.ClientRect;
   With FBackBuffer.Canvas do
   begin
      Brush.Color := clWhite;
      Pen.Color := clBlack;
      FillRect(r);
   end;
   FController.Render(FBackBuffer.Canvas);
   PB.Canvas.Draw(0, 0, FBackBuffer);
end;

procedure TMainWindow.FormKeyPress(Sender: TObject; var Key: Char);
begin
   Case Key of
   'f', 'F':   FController.FastRenderToggle;
   end;
end;

end.
