unit Matrix2dU;

interface

uses Classes, UtilsU;

Type
   TS2DMatrix = record
      _11: Double;
      _12: Double;
      _13: Double;
      _21: Double;
      _22: Double;
      _23: Double;
      _31: Double;
      _32: Double;
      _33: Double;
   end;

   T2DMatrix = class(TObject)
   private
      FMatrix: TS2DMatrix;

      procedure S2dMatrixMultiply(AMatrix: TS2DMatrix);
   public
      Constructor Create;

      procedure Identity;

      procedure Translate(x, y: Double);
      procedure Scale(xScale, yScale: Double);
      procedure Rotate(rotation: Double);

      procedure TransformPoints(var pv: Array of TSPoint);
   end;

implementation

{ T2DMatrix }

constructor T2DMatrix.Create;
begin
   FillChar(FMatrix, Sizeof(FMatrix), 0);
   Identity;
end;

procedure T2DMatrix.Identity;
begin
	FMatrix._11 := 1;
   FMatrix._12 := 0;
   FMatrix._13 := 0;

	FMatrix._21 := 0;
   FMatrix._22 := 1;
   FMatrix._23 := 0;

	FMatrix._31 := 0;
   FMatrix._32 := 0;
   FMatrix._33 := 1;
end;

procedure T2DMatrix.Rotate(rotation: Double);
var
   mat: TS2DMatrix;
   fSin, fCos: Double;
begin
   fSin := sin(rotation);
   fCos := cos(rotation);

	mat._11 := fCos;
   mat._12 := fSin;
   mat._13 := 0;

	mat._21 := -fSin;
   mat._22 := fCos;
   mat._23 := 0;

	mat._31 := 0;
   mat._32 := 0;
   mat._33 := 1;

   S2DMatrixMultiply(mat);
end;

procedure T2DMatrix.S2dMatrixMultiply(AMatrix: TS2DMatrix);
var
   mat_temp: TS2DMatrix;
begin
	//first row
	mat_temp._11 := (FMatrix._11 * AMatrix._11) + (FMatrix._12 * AMatrix._21) + (FMatrix._13 * AMatrix._31);
	mat_temp._12 := (FMatrix._11 * AMatrix._12) + (FMatrix._12 * AMatrix._22) + (FMatrix._13 * AMatrix._32);
	mat_temp._13 := (FMatrix._11 * AMatrix._13) + (FMatrix._12 * AMatrix._23) + (FMatrix._13 * AMatrix._33);

	//second
	mat_temp._21 := (FMatrix._21 * AMatrix._11) + (FMatrix._22 * AMatrix._21) + (FMatrix._23 * AMatrix._31);
	mat_temp._22 := (FMatrix._21 * AMatrix._12) + (FMatrix._22 * AMatrix._22) + (FMatrix._23 * AMatrix._32);
	mat_temp._23 := (FMatrix._21 * AMatrix._13) + (FMatrix._22 * AMatrix._23) + (FMatrix._23 * AMatrix._33);

	//third
	mat_temp._31 := (FMatrix._31 * AMatrix._11) + (FMatrix._32 * AMatrix._21) + (FMatrix._33 * AMatrix._31);
	mat_temp._32 := (FMatrix._31 * AMatrix._12) + (FMatrix._32 * AMatrix._22) + (FMatrix._33 * AMatrix._32);
	mat_temp._33 := (FMatrix._31 * AMatrix._13) + (FMatrix._32 * AMatrix._23) + (FMatrix._33 * AMatrix._33);

   FMatrix := mat_temp;
end;

procedure T2DMatrix.Scale(xScale, yScale: Double);
var
   mat: TS2DMatrix;
begin
	mat._11 := xScale;
   mat._12 := 0;
   mat._13 := 0;

	mat._21 := 0;
   mat._22 := yScale;
   mat._23 := 0;

	mat._31 := 0;
   mat._32 := 0;
   mat._33 := 1;

   S2DMatrixMultiply(mat);
end;

procedure T2DMatrix.Translate(x, y: Double);
var
   mat: TS2DMatrix;
begin
	mat._11 := 1;
   mat._12 := 0;
   mat._13 := 0;

	mat._21 := 0;
   mat._22 := 1;
   mat._23 := 0;

	mat._31 := x;
   mat._32 := y;
   mat._33 := 1;

   S2DMatrixMultiply(mat);
end;

procedure T2DMatrix.TransformPoints(var pv: Array of TSPoint);
var
   i: Integer;
   tempX: Double;
   tempY: Double;
begin
   For i := 0 to Length(pv) -1 do
   begin
      tempX := (FMatrix._11 * pv[i].x) + (FMatrix._21 * pv[i].y) + (FMatrix._31);
      TempY := (FMatrix._12 * pv[i].x) + (FMatrix._22 * pv[i].y) + (FMatrix._32);
      pv[i].x := tempX;
      pv[i].y := tempY;
   end;
end;

end.
