unit ParamsU;

interface

Type
   TParams = class(TObject)
   private
      FBias: Double;
      FActivationResponse: Double;
      FMutationRate: Double;
      FMaxPerturbation: Double;
      FCrossoverRate: Double;
      FMaxTurnRate: Double;
      FMaxSpeed: Double;
      FHalfPi: Double;
      FTwoPi: Double;
      FMineScale: Double;
      FPi: Double;
      FNumHidden: Integer;
      FWindowWidth: Integer;
      FNumMines: Integer;
      FNumInputs: Integer;
      FFramesPerSecond: Integer;
      FNeuronsPerHiddenLayer: Integer;
      FWindowHeight: Integer;
      FNumElite: Integer;
      FNumTicks: Integer;
      FSweeperScale: Integer;
      FNumCopiesElite: Integer;
      FNumOutputs: Integer;
      FNumSweepers: Integer;
      procedure LoadParams;
   public
      Constructor Create;

      //------------------------------------general parameters
      property Pi: Double read FPi;
      property HalfPi: Double read FHalfPi;
      property TwoPi: Double read FTwoPi;

      property WindowWidth: Integer read FWindowWidth;
      property WindowHeight: Integer read FWindowHeight;
      property FramesPerSecond: Integer read FFramesPerSecond;

      //-------------------------------------used for the neural network
      property NumInputs: Integer read FNumInputs;
      property NumHidden: Integer read FNumHidden;
      property NeuronsPerHiddenLayer: Integer read FNeuronsPerHiddenLayer;
      property NumOutputs: Integer read FNumOutputs;

      //for tweeking the sigmoid function
      property ActivationResponse: Double read FActivationResponse;

      //bias value
      property Bias: Double read FBias;

      //--------------------------------------used to define the sweepers

      //limits how fast the sweepers can turn
      property MaxTurnRate: Double read FMaxTurnRate;
      property MaxSpeed: Double read FMaxSpeed;

      //for controlling the size
      property SweeperScale: Integer read FSweeperScale;

      //--------------------------------------controller parameters
      property NumSweepers: Integer read FNumSweepers;
      property NumMines: Integer read FNumMines;

      //number of time steps we allow for each generation to live
      property NumTicks: Integer read FNumTicks;

      //scaling factor for mines
      property MineScale: Double read FMineScale;

      //---------------------------------------GA parameters
      property CrossoverRate: Double read FCrossoverRate;
      property MutationRate: Double read FMutationRate;

      //the maximum amount the ga may mutate each weight by
      property MaxPerturbation: Double read FMaxPerturbation;

      //used for elitism
      property NumElite: Integer read FNumElite;
      property NumCopiesElite: Integer read FNumCopiesElite;
   end;

Var
   Params: TParams;

implementation

uses SysUtils, Forms, IniFiles;

{ TParams }

constructor TParams.Create;
begin
   LoadParams;
end;

procedure TParams.LoadParams;
var
   FileName: String;
begin
   FileName := ExtractFilePath(Application.ExeName) + '\Params.ini';
   if not FileExists(FileName) then
      Raise Exception.Create('Couldn''t find the ini file!!!!');

   With TIniFile.Create(FileName) do
   try
      FFramesPerSecond        :=   ReadInteger('Settings', 'iFramesPerSecond', 0);
      FNumInputs              :=   ReadInteger('Settings', 'iNumInputs', 0);
      FNumHidden              :=   ReadInteger('Settings', 'iNumHidden', 0);
      FNeuronsPerHiddenLayer  :=   ReadInteger('Settings', 'iNeuronsPerHiddenLayer', 0);
      FNumOutputs             :=   ReadInteger('Settings', 'iNumOutputs', 0);
      FActivationResponse     :=   ReadFloat('Settings', 'dActivationResponse', 0.0);
      FBias                   :=   ReadFloat('Settings', 'dBias', 0.0);;
      FMaxTurnRate            :=   ReadFloat('Settings', 'dMaxTurnRate', 0.0);;
      FMaxSpeed               :=   ReadFloat('Settings', 'dMaxSpeed', 0.0);;
      FSweeperScale           :=   ReadInteger('Settings', 'iSweeperScale', 0);
      FNumMines               :=   ReadInteger('Settings', 'iNumMines', 0);
      FNumSweepers            :=   ReadInteger('Settings', 'iNumSweepers', 0);
      FNumTicks               :=   ReadInteger('Settings', 'iNumTicks', 0);
      FMineScale              :=   ReadFloat('Settings', 'dMineScale', 0.0);;
      FCrossoverRate          :=   ReadFloat('Settings', 'dCrossoverRate', 0.0);;
      FMutationRate           :=   ReadFloat('Settings', 'dMutationRate', 0.0);;
      FMaxPerturbation        :=   ReadFloat('Settings', 'dMaxPerturbation', 0.0);;
      FNumElite               :=   ReadInteger('Settings', 'iNumElite', 0);
      FNumCopiesElite         :=   ReadInteger('Settings', 'iNumCopiesElite', 0);
   finally
      Free;
   end;

   FPi := Pi;
   FHalfPi := Pi / 2;
   FTwoPi := Pi * 2;

   FWindowWidth := 400;
   FWindowHeight := 400;
end;

initialization
   Params := TParams.Create;

finalization
   Params.Free;

end.
