unit S2DVectorU;

interface

uses Contnrs;

Type
   TSVector2D = class(TObject)
   private
      FY: Double;
      FX: Double;
      procedure SetX(const Value: Double);
      procedure SetY(const Value: Double);
      function GetLength: Double;

   public
      Constructor Create(AX: Double = 0; AY: Double = 0);

      procedure AddVector(AVector: TSVector2D);
      procedure Assign(AVector: TSVector2D);
      procedure SubVector(AVector: TSVector2D);
      procedure Multiply(AVal: Double);
      procedure Divide(AVal: Double);
      function  Dot(AVector: TSVector2D): Double;

      function  Sign(AVector: TSVector2D): Integer;

      procedure Normalize;

      property X: Double read FX write SetX;
      property Y: Double read FY write SetY;
      property Length: Double read GetLength;
   end;

   TSVector2DList = class(TObject)
   private
      FVectors: TObjectList;
      function GetCount: Integer;
      function GetVectors(Idx: Integer): TSVector2D;

   public
      Constructor Create;
      Destructor Destroy; Override;

      function Add: TSVector2D;
      procedure Clear;
      procedure Remove(Idx: Integer);

      property Count: Integer read GetCount;
      property Vectors[Idx: Integer]: TSVector2D read GetVectors; Default;
   end;

implementation

{ TSVector2D }

procedure TSVector2D.AddVector(AVector: TSVector2D);
begin
   FX := FX + AVector.X;
   FY := FY + AVector.Y;
end;

procedure TSVector2D.Assign(AVector: TSVector2D);
begin
   FX := AVector.X;
   FY := AVector.Y;
end;

constructor TSVector2D.Create(AX: Double = 0; AY: Double = 0);
begin
   FX := AX;
   FY := AY;
end;

procedure TSVector2D.Divide(AVal: Double);
begin
   FX := FX / AVal;
   FY := FY / AVal;
end;

function TSVector2D.Dot(AVector: TSVector2D): Double;
begin
   Result := FX * AVector.X + FY * AVector.Y;
end;

function TSVector2D.GetLength: Double;
begin
   Result := Sqrt(FX * FX + FY * FY);
end;

procedure TSVector2D.Multiply(AVal: Double);
begin
   FX := FX * AVal;
   FY := FY * AVal;
end;

procedure TSVector2D.Normalize;
begin
   FX := FX / Length;
   FY := FY / Length;
end;

procedure TSVector2D.SetX(const Value: Double);
begin
   FX := Value;
end;

procedure TSVector2D.SetY(const Value: Double);
begin
   FY := Value;
end;

function TSVector2D.Sign(AVector: TSVector2D): Integer;
begin
   //  returns positive if AVector is clockwise of Self, minus if counter-clockwise
   if FY * AVector.X > FX * AVector.Y then
      Result := 1
   else
      Result := -1;
end;

procedure TSVector2D.SubVector(AVector: TSVector2D);
begin
   FX := FX - AVector.X;
   FY := FY - AVector.Y;
end;

{ TSVector2DList }

function TSVector2DList.Add: TSVector2D;
begin
   Result := TSVector2D.Create;
   FVectors.Add(Result);
end;

procedure TSVector2DList.Clear;
begin
   FVectors.Clear;
end;

constructor TSVector2DList.Create;
begin
   FVectors := TObjectList.Create;
end;

destructor TSVector2DList.Destroy;
begin
   FVectors.Free;
   inherited;
end;

function TSVector2DList.GetCount: Integer;
begin
   Result := FVectors.Count;
end;

function TSVector2DList.GetVectors(Idx: Integer): TSVector2D;
begin
   Result := TSVector2D(FVectors[Idx]);
end;

procedure TSVector2DList.Remove(Idx: Integer);
begin
   FVectors.Delete(Idx);
end;

end.
