unit TimerU;

interface

Type
   TMSTimer = class(TObject)
   private
      FCurrentTime: Int64;
      FLastTime: Int64;
      FNextTime: Int64;
      FFrameTime: Int64;
      FPerfCountFreq: Int64;
      FTimeElapsed: Double;
      FTimeScale: Double;

      FFPS: Double;
      function GetReadyForNextFrame: Boolean;
   public
      Constructor Create(AFPS: Double); Overload;

      procedure Start;

      property ReadyForNextFrame: Boolean read GetReadyForNextFrame;
   end;

implementation

uses Windows, SysUtils;

{ TMSTimer }

constructor TMSTimer.Create(AFPS: Double);
begin
   FFPS := AFPS;
   FTimeElapsed := 0;
   FLastTime := 0;
   FFrameTime := 0;
   FPerfCountFreq := 0;

	//how many ticks per sec do we get
	QueryPerformanceFrequency(FPerfCountFreq);

   FTimeScale := 1.0 / FPerfCountFreq;

	//calculate ticks per frame
   FFrameTime := FPerfCountFreq div Round(FFPS);
end;

function TMSTimer.GetReadyForNextFrame: Boolean;
begin
   Result := False;

   if FFPS = 0 then
   begin
      raise Exception.Create('No FPS set in timer, "Doh!"');
   end;

   QueryPerformanceCounter(FCurrentTime);

   if FCurrentTime > FNextTime then
   begin
      FTimeElapsed := (FCurrentTime - FLastTime) * FTimeScale;
      FLastTime := FCurrentTime;

      FNextTime := FCurrentTime + FFrameTime;
      Result := True;
   end;
end;

procedure TMSTimer.Start;
begin
	//get the time
	QueryPerformanceCounter(FLastTime);

	//update time to render next frame
   FNextTime := FLastTime + FFrameTime;
end;

end.
